/*
Copyright (c) 1998 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version. This program is
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// Atari810.cpp : Defines the entry point for the console application.
//

#ifdef WIN32
#include "..\\win32a\\stdafx.h"
#else if __GNUC__
#include "gnuDefines.h"
#endif
#include "innerShell.h"

#include <conio.h>

void showUsage();
void showBlurb();

int main(int argc, char* argv[])
{
	LPSTR prot=NULL;
	BYTE comPort=0;
	bool debugEnabled = false;
	bool traceEnabled = false;
	bool logEnabled = false;

	DWORD diskid=1;
	int ret=0;
	int i;

	printf("\nAtari810 1.0, April 6 2001\n");
	printf("by Dan Vernon, dan@bitsofthepast.com\n");

	if (argc==1)
	{
		showUsage();
		showBlurb();
		return 0;
	}

	try
	{
		//process options
		for(i=1;i<argc;i++)
			if (argv[i][0]=='-' || argv[i][0]=='/')
				switch(tolower(argv[i][1]))
				{
				case 'p':
					prot = &(argv[i][2]);
				break;
				case 'c':
					comPort = argv[i][2]-'0';
				break;
				case 'd':
					debugEnabled=true;
				break;
				case 't':
					traceEnabled=true;
				break;
				case 'l':
					logEnabled=true;
				break;
				default:
					printf("ignoring invalid option: %s\n",argv[i]);
				}

		if(!comPort) comPort=1;
		innerShell_init();
		innerShell_setSerialPort(comPort);

		//process disk image arguments
		for(i=1;i<argc;i++)
			if (argv[i][0]!='-' && argv[i][0]!='/')
			{
				if (strcmp(argv[i],"."))
				{
					innerShell_mountDisk(argv[i],diskid,prot?strchr(prot,'0'+i)!=NULL:false);
				}
				diskid++;
			}

		printf("\n"); //dump a blank line before starting

		//start up the sio emulator
		innerShell_start();

		//wait for thread to stop on cancel or error
		while(!innerShell_isStopped())
		{
			Sleep(300);
			innerShell_checkClientCancel();
		}
	}
	catch(...){}

	// dump out any leftover messages
	innerShell_emptyEventQueue();

	if(innerShell_isErrorSet())
	{
		printf("\nabnormal termination\n");
		ret=1;
	}

	innerShell_destroy();

	printf("\n[press any key]");
	getch();
	printf("\n");
	return ret;
}

void showUsage()
{
	printf("%s"," \
\n \
usage: Atari810 [options] atrfile...\n \
options: \n \
-p<disk id list>       opens the disk ids (i.e. 1-9) as read-only\n \
-c<port number>        serial port having a SIO2PC cable (defaults to 1)\n \
\n \
up to 9 atr disk images can be passed in and will be mounted as atari\n \
drives in ascending order starting from D1.  To skip a drive, just use\n \
a single period (.) as an argument. To exit the program, press [Esc].\n \
\n \
hint: To save typing, it is possible from Explorer to drag and drop\n \
several images at once onto the Atari810.exe to mount them.\n \
If you need a different serial port than the default, you can write a\n \
batch file that calls Atari810 and just drag and drop the images \n \
onto the batch file instead.\n \
\n \
sample batch file: (assuming your SIO2PC cable is on COM2)\n \
\\SomeFolder\\Atari810.exe -c2 -p789 %1 %2 %3 %4 %5 %6 %7 %8 %9 \n \
\n \
\n \
\n \
[press any key]");
	getch();
}

void showBlurb()
{
	printf("\n \
\n \
\n \
Keep in mind that although this is free software, I may need a little en-\n \
couragement from time to time to keep working on new features and fixing\n \
the bugs! (hint: donating an Atari 8-bit cart would be greatly appreciated)\n \
My plans are to continue to improve Atari810.  You can find the most cur-\n \
rent version at http://www.bitsofthepast.com/atari.\n \
\n \
You can mail me with questions or suggestions; however, I have a wife and\n \
four small children, plus a full-time real job, so please be patient.\n \
\n \
contact information:\n \
\n \
email: dan@bitsofthepast.com   -or-      snail mail: Dan Vernon\n \
                                                     Safeco Plaza\n \
                                                     Seattle WA 98185\n \
\n \
\n \
\n \
\n \
\n \
\n \
\n \
\n \
[press any key]");
	getch();
}