/*
Copyright (c) 2001 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version. This program is
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// InnerShell.h
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_INNERSHELL_H__E4DFCDB3_1298_4B17_91ED_F1AD60C12FB8__INCLUDED_)
#define AFX_INNERSHELL_H__E4DFCDB3_1298_4B17_91ED_F1AD60C12FB8__INCLUDED_

#ifdef WIN32
#include "winDefines.h"
#include "stdafx.h"
#include "atariSioDriver.h"
#else if __GNUC__
#include "gnuDefines.h"
#endif

#include "Shell.h"

#include "sioProtocol.h"
#include "stateStore.h"
#include "eventQueue.h"

#define VERSION "1.4d"

#define MSG_SIZE 500

class CSIOProtocol;
class CStateStore;
class CEventQueue;
class Shell;

extern CSIOProtocol* g_sio;
extern CStateStore* g_state;
extern CEventQueue* g_evq;
extern Shell* g_sh;
extern bool g_shownVersion;


void innerShell_init(Shell* sh);
void innerShell_destroy();

bool innerShell_createImage(LPSTR imagepath,WORD imageSectorCount, WORD imageSectorSize);
bool innerShell_mount(LPSTR path,BYTE sioid,bool readOnly);
bool innerShell_release(BYTE sioid);
bool innerShell_setSerialPort(BYTE portNumber);

void innerShell_start();
void innerShell_stop();
bool innerShell_isStopped();
bool innerShell_isStarted();

bool innerShell_isErrorSet();
bool innerShell_isCancelSet();
void innerShell_setError(bool flag);
void innerShell_setCancel(bool flag);

void innerShell_checkClientCancel();
void innerShell_emptyEventQueue();

void innerShell_debug(LPCSTR fmt, ...);
void innerShell_status(LPCSTR fmt, ...);
void innerShell_error(DWORD number, LPCSTR src, LPCSTR func, LPCSTR fmt, ...);
void innerShell_criticalSioError(DWORD number, LPCSTR src, LPCSTR func, LPCSTR fmt, ...);
void innerShell_sioCommand(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex);
void innerShell_debugSioCommand(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex);

void innerShell_displayStatus(LPCSTR msg);
void innerShell_displayDebug(LPCSTR msg);
void innerShell_displayError(DWORD number, LPCSTR msg);
void innerShell_displayCommand(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex);

LPCSTR innerShell_getDiskPath(BYTE diskid);
void innerShell_swapDisks(BYTE diskid1,BYTE diskid2);

bool innerShell_getDiskReadOnly(BYTE diskid);
WORD innerShell_getDiskSectorSize(BYTE diskid);
WORD innerShell_getDiskSectorCount(BYTE diskid);

BYTE innerShell_getMaxSpeed();
void innerShell_putMaxSpeed(BYTE newval);

bool innerShell_getShowDebugMessages();
void innerShell_putShowDebugMessages(bool newval);

#endif // !defined(AFX_INNERSHELL_H__E4DFCDB3_1298_4B17_91ED_F1AD60C12FB8__INCLUDED_)
