/*
Copyright (c) 2001 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version. This program is
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// sioPort.cpp: implementation of the CSIOPort class.
//
//////////////////////////////////////////////////////////////////////

#ifdef WIN32
#include "windefines.h"
#else if __GNUC__
#include "gnuDefines.h"
#endif
#include "sioPort.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSIOPort::CSIOPort()
{
	handleCom=NULL;
	overlapReadWrite.hEvent=NULL;
	overlapRing.hEvent=NULL;
	portNumber=0;
	portOpened=false;
	maxSpeed=3;
}

CSIOPort::~CSIOPort()
{
	closePort();
}

bool CSIOPort::setSerialPort(BYTE portNumber)
{
	if (portNumber!=this->portNumber)
		openPort(portNumber);
	return (handleCom!=NULL);
}

void CSIOPort::reportPortStatus(DWORD eventMask)
{
    DWORD fRING, fRLSD, fRXCHAR, fRXFLAG, fTXEMPTY;
    DWORD fBREAK, fCTS, fDSR, fERR, fRX80FULL;
    char szMessage[500];

    fCTS = EV_CTS & eventMask;
    fDSR = EV_DSR & eventMask;
    fERR = EV_ERR & eventMask;
    fRING = EV_RING & eventMask;
    fRLSD = EV_RLSD & eventMask;
    fBREAK = EV_BREAK & eventMask;
    fRXCHAR = EV_RXCHAR & eventMask;
    fRXFLAG = EV_RXFLAG & eventMask;
    fTXEMPTY = EV_TXEMPTY & eventMask;
    fRX80FULL = EV_RX80FULL & eventMask;

	szMessage[0]=NULL;
    strcat(szMessage, fCTS ? "cts " : "");
    strcat(szMessage, fDSR ? "dsr " : "");
    strcat(szMessage, fERR ? "err " : "");
    strcat(szMessage, fRING ? "ring " : "");
    strcat(szMessage, fRLSD ? "rlsd " : "");
    strcat(szMessage, fBREAK ? "break " : "");
    strcat(szMessage, fRXFLAG ? "rxflag " : "");
    strcat(szMessage, fRXCHAR ? "rxchar " : "");
    strcat(szMessage, fTXEMPTY ? "txempty " : "");
    strcat(szMessage, fRX80FULL ? "rx80full " : "");

	innerShell_debug("%s",szMessage);
}

bool CSIOPort::openPort(BYTE portNumber)
{
	LPCSTR src = "CSIOPort::openPort";
	COMMTIMEOUTS mTimeouts = { 0, 0, 0, 0, 0 };

	if (portOpened) closePort();

	char portName[5];
	portName[0]='C';
	portName[1]='O';
	portName[2]='M';
	portName[3]='0'+portNumber;
	portName[4]=NULL;

	handleCom = CreateFile( portName,
		GENERIC_READ | GENERIC_WRITE,
		0,
		NULL,
		OPEN_EXISTING,
		FILE_FLAG_OVERLAPPED,
		NULL
		);

	if (handleCom == INVALID_HANDLE_VALUE)
	{
		innerShell_error(GetLastError(),src,"CreateFile",NULL);
		return false;
	}

    DCB dcb = {0};

    dcb.DCBlength = sizeof(dcb);

    //
    // get current DCB settings
    //
    if (!GetCommState(handleCom, &dcb))
	{
		innerShell_error(GetLastError(),src,"GetCommState",NULL);
		return false;
	}

    //
    // update DCB rate, byte size, parity, and stop bits size
    //
    dcb.BaudRate = CBR_19200;
    dcb.ByteSize = 8;
    dcb.Parity   = NOPARITY;
    dcb.StopBits = ONESTOPBIT;

    //
    // update event flags
    //
	dcb.EvtChar = '\0';

    //
    // update flow control settings
    //
    dcb.fDtrControl     = DTR_CONTROL_DISABLE; // DTR_CONTROL_ENABLE;
    dcb.fRtsControl     = RTS_CONTROL_DISABLE; //RTS_CONTROL_ENABLE;

    dcb.fOutxCtsFlow    = FALSE;
    dcb.fOutxDsrFlow    = FALSE;
    dcb.fDsrSensitivity = FALSE;
    dcb.fOutX           = FALSE;
    dcb.fInX            = FALSE;
    dcb.fTXContinueOnXoff = FALSE;
    dcb.XonChar         = ASCII_XON;
    dcb.XoffChar        = ASCII_XOFF;
    dcb.XonLim          = 0;
    dcb.XoffLim         = 0;

	dcb.fBinary = 1;
	dcb.fAbortOnError=1;

    //
    // DCB settings not in the user's control
    //
    dcb.fParity = TRUE;


    // set new state
    if (!SetCommState(handleCom, &dcb))
	{
		innerShell_error(GetLastError(),src,"SetCommState",NULL);
		return false;
	}

	// set new timeouts
    if (!SetCommTimeouts(handleCom, &mTimeouts))
	{
		innerShell_error(GetLastError(),src,"SetCommTimeouts",NULL);
		return false;
	}

    if(!SetupComm(handleCom, MAX_READ_BUFFER, MAX_WRITE_BUFFER))
	{
		innerShell_error(GetLastError(),src,"SetupComm",NULL);
		return false;
	}

	// Set the event mask.
	if (!SetCommMask(handleCom, EV_RING))
	{
		innerShell_error(GetLastError(),src,"SetCommMask",NULL);
		return false;
	}

	overlapRing.hEvent = CreateEvent(
		NULL,   // no security attributes
		FALSE,  // auto reset event
		FALSE,  // not signaled
		NULL    // no name
		);
	if(!overlapRing.hEvent)
		innerShell_error(GetLastError(),src,"CreateEvent",NULL);

	overlapReadWrite.hEvent = CreateEvent(
		NULL,   // no security attributes
		FALSE,  // auto reset event
		FALSE,  // not signaled
		NULL    // no name
		);
	if(!overlapReadWrite.hEvent)
		innerShell_error(GetLastError(),src,"CreateEvent",NULL);

	this->portNumber=portNumber;
	this->portOpened=true;
	return this->portOpened;
}

bool CSIOPort::setSpeed(DWORD baudRate)
{
/*
#define CBR_38400           38400
#define CBR_56000           56000
#define CBR_57600           57600
*/
	LPCSTR src = "CSIOPort::setSpeed";
    DCB dcb = {0};

    //
    // get current DCB settings
    //
	ClearCommError(handleCom,0,0);  //just in case

    if (!GetCommState(handleCom, &dcb))
		innerShell_criticalSioError(GetLastError(),src,"GetCommState",NULL);

	dcb.BaudRate = baudRate;

    // set new state
    if (!SetCommState(handleCom, &dcb))
		innerShell_criticalSioError(GetLastError(),src,"SetCommState",NULL);

	currentBaud = baudRate;
	return true;
}


bool CSIOPort::clearError()
{
	DWORD err;

	err=ClearCommError(handleCom,0,0);

	// in case the hardware detects a framing error or an overflow,
	// toggle the baud rate (if speed-up is active).
	if(CE_FRAME==err || CE_RXOVER==err)
	{
		innerShell_debug("comm error: frame/overflow error");
		if (1 || g_state->speedUpActive)
		{
			if (currentBaud==CBR_19200 && maxSpeed!=1)
			{
				setSpeed((maxSpeed==2)?CBR_38400:CBR_57600);
				g_state->setCurrentSpeed(maxSpeed);

			}
			else
			{
				setSpeed(CBR_19200);
				g_state->setCurrentSpeed(1);
			}
		}
		return false;
	}
	else
	{
		innerShell_debug("comm error #: %d",err);
	}
	return true;
}

void CSIOPort::closePort()
{
	if(handleCom)
		CloseHandle(handleCom);

	if(overlapRing.hEvent)
		CloseHandle(overlapRing.hEvent);

	if(overlapReadWrite.hEvent)
		CloseHandle(overlapReadWrite.hEvent);

	portOpened=false;
	portNumber=0;
}

//whenever we get checksum errors, nuke the i/o buffers & start over
void CSIOPort::resetPort()
{
	PurgeComm(handleCom,PURGE_RXCLEAR); //empty the input buffer
	PurgeComm(handleCom,PURGE_TXCLEAR); //empty the output buffer
}

bool CSIOPort::doOverlappedComCrud(DWORD func, DWORD count, BYTE* chunk)
{
	LPCSTR src = "CSIOPort::doOverlappedComCrud";
	bool waitingRing = false;
	bool waitingIO = false;
	bool overlappedFinished = false;
	DWORD eventMask;
	DWORD res=0;
	DWORD bytesCompleted=0;
	DWORD totalBytesCompleted=0;
	DWORD ringResult=0;
	bool ringHappened=false;
	DWORD lastError=0;
	LPCSTR funcName=NULL;
	bool success = true;
	bool waitingForCommand = (count==5 && func==SERIAL_API_READ);

	HANDLE h[4];
	h[0]=overlapRing.hEvent;
	h[1]=overlapReadWrite.hEvent;
	h[2]=g_state->cancelEvent;
	h[3]=g_state->errorEvent;


	while(!overlappedFinished && !g_state->isErrorSet())
	{
		if (!waitingRing)
		{
			overlapRing.Offset=0;
			overlapRing.OffsetHigh=0;
			ResetEvent(overlapRing.hEvent);

			ringInterruptFlag=false;

 			res=WaitCommEvent(handleCom, &eventMask, &overlapRing);
			if (res)
			{
				if (eventMask & EV_RING)
				{
					ringInterruptFlag=true;
					overlappedFinished=true;
					success=false;  //indicates to AtariSIO to read next command
				}
			}
			else
			{
				lastError = GetLastError();
				switch(lastError)
				{
				case ERROR_IO_PENDING:
					waitingRing=true;
					break;
				case ERROR_OPERATION_ABORTED:
					clearError();
					//just try again
					break;
				default:
					innerShell_criticalSioError(GetLastError(),src,"WaitCommEvent",NULL);
				}
			}
		}

		if (waitingRing && !waitingIO)
		{
			overlapReadWrite.Offset=0;
			overlapReadWrite.OffsetHigh=0;
			ResetEvent(overlapReadWrite.hEvent);

			switch(func)
			{
			case SERIAL_API_READ:
				res=ReadFile(handleCom, &chunk[totalBytesCompleted], count-totalBytesCompleted, &bytesCompleted, &overlapReadWrite);
				funcName="ReadFile";
				break;
			case SERIAL_API_WRITE:
				res=WriteFile(handleCom, &chunk[totalBytesCompleted], count-totalBytesCompleted, &bytesCompleted, &overlapReadWrite);
				funcName="WriteFile";
				break;
			}

			if (res)
				overlappedFinished = (bytesCompleted == count);
			else
			{
				lastError = GetLastError();
				switch(lastError)
				{
				case ERROR_IO_PENDING:
					waitingIO=true;
					break;
				case ERROR_OPERATION_ABORTED:
					if(!clearError())
					{
						resetPort();
						overlappedFinished=true;
						success=false;  //indicates to AtariSIO to read next command
					}
					else
						waitingIO=false; //try again
					break;
				default:
					innerShell_criticalSioError(GetLastError(),src,funcName,NULL);
				}
			}
		}

		if (waitingRing || waitingIO)
		{
			res = WaitForMultipleObjects(4,h,false,INFINITE);
			switch(res)
			{
			case WAIT_TIMEOUT:
				// should never be here!
				break;

			case WAIT_FAILED:
				// should never be here either, for that matter.
				innerShell_criticalSioError(GetLastError(),src,"WaitForSingleObject",NULL);
				break;

			case WAIT_OBJECT_0: //ring
				waitingRing=false;
				if (!GetOverlappedResult(handleCom, &overlapRing, &ringResult, FALSE))
				{
					if (GetLastError() == ERROR_OPERATION_ABORTED)
					{
						clearError();
					}
					else
						innerShell_criticalSioError(GetLastError(),src,"GetOverlappedResult",NULL);
				}
                else
				{
					//reportPortStatus(eventMask);
					if (eventMask & EV_RING)
					{
						ringHappened=true;
						//if waiting on a command, just keep going
						if (waitingForCommand)
						{
							ResetEvent(overlapRing.hEvent);
						}
						else  //otherwise forget about what we were just doing
						{
							ringInterruptFlag=true;
							overlappedFinished=true;
							success=false;  //indicates to AtariSIO to read next command
						}
					}
				}
				break;

			case WAIT_OBJECT_0+1: // read or write
				if (waitingForCommand && !ringHappened)
				{
					resetPort();
					overlappedFinished=true;
					success=false;  //indicates to AtariSIO to read next command
				}
				else
				{
					waitingIO=false;
					if (!GetOverlappedResult(handleCom, &overlapReadWrite, &bytesCompleted, FALSE)) {
						if (GetLastError() == ERROR_OPERATION_ABORTED)
						{
							//innerShell_debug("(%s!)",func==SERIAL_API_READ?"r":"w");
							if(!clearError())
							{
								resetPort();
								overlappedFinished=true;
								success=false;  //indicates to AtariSIO to read next command
							}
						}
						else
							innerShell_criticalSioError(GetLastError(),src,"GetOverlappedResult",NULL);
					}
					else
					{
						totalBytesCompleted+=bytesCompleted;
						overlappedFinished = (totalBytesCompleted == count);
					}
				}
				break;

			case WAIT_OBJECT_0+2: // cancel
			case WAIT_OBJECT_0+3: // error
				success=false;
				overlappedFinished = true;
				break;
			}
		}
	}
	CancelIo(handleCom);  //cancels any pending IO on the port
	return (success && !g_state->isErrorSet());
}

bool CSIOPort::readBytes(BYTE* chunk,DWORD ct)
{
	return doOverlappedComCrud(SERIAL_API_READ,ct,chunk);
}

bool CSIOPort::writeBytes(BYTE* chunk,DWORD ct)
{
	return doOverlappedComCrud(SERIAL_API_WRITE,ct,chunk);
}

bool CSIOPort::writeAck(BYTE byte)
{
	return doOverlappedComCrud(SERIAL_API_WRITE,1,&byte);
}
