/*
Copyright (c) 2001 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version. This program is
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// Shell.cpp : Implementation of the Shell class

#include "stdafx.h"
#include "atariSioDriver.h"
#include "Shell.h"

#include "innerShell.h"

/////////////////////////////////////////////////////////////////////////////
// Shell

Shell::Shell()
{
	innerShell_init(this);
}
Shell::~Shell()
{
	innerShell_destroy();
}

/////////////////////////////////////////////////////////////////////////////
// Event firing methods (called by innerShell)
//
void Shell::Command(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command,BYTE speedIndex)
{
	Fire_Command(b1,b2,b3,b4,b5,disk,(LONG)sector,CComBSTR(command),speedIndex);
}
void Shell::Status(LPCSTR msg)
{
	Fire_Status(CComBSTR(msg));
}
void Shell::Debug(LPCSTR msg)
{
	Fire_Debug(CComBSTR(msg));
}
void Shell::Error(LONG number, LPCSTR msg)
{
	Fire_Error(number,CComBSTR(msg));
}


/////////////////////////////////////////////////////////////////////////////
// Support fancy COM errors
//
STDMETHODIMP Shell::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] =
	{
		&IID_IShell
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// COM methods
//
STDMETHODIMP Shell::mount(BSTR path, BYTE diskid, VARIANT_BOOL readOnly)
{
	USES_CONVERSION;

	LPSTR tempPath = W2A(path);

	innerShell_mount(tempPath,diskid,readOnly!=0);
	return S_OK;
}

STDMETHODIMP Shell::release(BYTE diskid)
{
	innerShell_release(diskid);
	return S_OK;
}

STDMETHODIMP Shell::start()
{
	innerShell_start();
	return S_OK;
}

STDMETHODIMP Shell::stop()
{
	innerShell_stop();
	innerShell_emptyEventQueue();
	return S_OK;
}

STDMETHODIMP Shell::get_isStopped(VARIANT_BOOL *pVal)
{
	*pVal =  innerShell_isStopped();
	return S_OK;
}

STDMETHODIMP Shell::get_serialPort(BYTE *pVal)
{
	*pVal = g_sio->sioPort->portNumber;
	return S_OK;
}

STDMETHODIMP Shell::put_serialPort(BYTE newVal)
{
	innerShell_setSerialPort(newVal);
	return S_OK;
}

STDMETHODIMP Shell::swapDisks(BYTE diskid1, BYTE diskid2)
{
	innerShell_swapDisks(diskid1,diskid2);
	return S_OK;
}

STDMETHODIMP Shell::get_diskPath(BYTE id, BSTR *pVal)
{
	CComBSTR bs;
	LPCSTR path = innerShell_getDiskPath(id);
	if(!path) path="";
	bs.Append(path);
	*pVal=bs.Detach();
	return S_OK;
}

STDMETHODIMP Shell::get_diskReadOnly(BYTE id, VARIANT_BOOL *pVal)
{
	*pVal=innerShell_getDiskReadOnly(id);
	return S_OK;
}

STDMETHODIMP Shell::get_diskSectorSize(BYTE id, LONG *pVal)
{
	*pVal=innerShell_getDiskSectorSize(id);
	return S_OK;
}

STDMETHODIMP Shell::get_diskSectorCount(BYTE id, LONG *pVal)
{
	*pVal=innerShell_getDiskSectorCount(id);
	return S_OK;
}

STDMETHODIMP Shell::createImage(BSTR imagePath, LONG sectorCount, LONG sectorSize)
{
	USES_CONVERSION;

	LPSTR tempPath = W2A(imagePath);

	innerShell_createImage(tempPath,LOWORD(sectorCount),LOWORD(sectorSize));

	return S_OK;
}

STDMETHODIMP Shell::get_versionString(BSTR *pVal)
{
	*pVal=CComBSTR(VERSION);
	return S_OK;
}

STDMETHODIMP Shell::get_maxSpeed(BYTE *pVal)
{
	*pVal=innerShell_getMaxSpeed();
	return S_OK;
}

STDMETHODIMP Shell::put_maxSpeed(BYTE newVal)
{
	innerShell_putMaxSpeed(newVal);
	return S_OK;
}

STDMETHODIMP Shell::get_showDebugMessages(VARIANT_BOOL* pVal)
{
	*pVal=innerShell_getShowDebugMessages();
	return S_OK;
}

STDMETHODIMP Shell::put_showDebugMessages(VARIANT_BOOL newVal)
{
	innerShell_putShowDebugMessages(newVal!=0);
	return S_OK;
}
