/*
Copyright (c) 2001 Dan Vernon

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version. This program is
distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even
the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details. You should have received a copy of the GNU
General Public License along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// Shell.h : Declaration of the Shell

#ifndef __Shell_H_
#define __Shell_H_

#include "resource.h"       // main symbols

#include "atariSioDriverCP.h"

/////////////////////////////////////////////////////////////////////////////
// Shell
class ATL_NO_VTABLE Shell : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<Shell, &CLSID_Shell>,
	public ISupportErrorInfo,
	public IConnectionPointContainerImpl<Shell>,
	public IDispatchImpl<IShell, &IID_IShell, &LIBID_atariSioDriver>,
	public CProxy_IShellEvents< Shell >
{
public:
	Shell();
	~Shell();
	void Debug(LPCSTR msg);
	void Status(LPCSTR msg);
	void Command(BYTE b1,BYTE b2,BYTE b3,BYTE b4,BYTE b5,BYTE disk,WORD sector,LPCSTR command, BYTE speedIndex);
	void Error(LONG number, LPCSTR msg);


DECLARE_REGISTRY_RESOURCEID(IDR_Shell)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(Shell)
	COM_INTERFACE_ENTRY(IShell)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(IConnectionPointContainer)
	COM_INTERFACE_ENTRY_IMPL(IConnectionPointContainer)
END_COM_MAP()
BEGIN_CONNECTION_POINT_MAP(Shell)
CONNECTION_POINT_ENTRY(DIID__IShellEvents)
END_CONNECTION_POINT_MAP()


// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

// IShell

public:
	STDMETHOD(get_maxSpeed)(/*[out, retval]*/ BYTE *pVal);
	STDMETHOD(put_maxSpeed)(/*[in]*/ BYTE newVal);
	STDMETHOD(get_versionString)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(createImage)(BSTR imagePath, LONG sectorCount, LONG sectorSize);
	STDMETHOD(get_diskSectorCount)(BYTE id, /*[out, retval]*/ LONG *pVal);
	STDMETHOD(get_diskSectorSize)(BYTE id, /*[out, retval]*/ LONG *pVal);
	STDMETHOD(get_diskReadOnly)(BYTE id, /*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(get_diskPath)(BYTE id, /*[out, retval]*/ BSTR *pVal);
	STDMETHOD(swapDisks)(BYTE diskid1, BYTE diskid2);
	STDMETHOD(get_serialPort)(/*[out, retval]*/ BYTE *pVal);
	STDMETHOD(put_serialPort)(/*[in]*/ BYTE newVal);
	STDMETHOD(get_isStopped)(/*[out, retval]*/ VARIANT_BOOL *pVal);
	STDMETHOD(stop)();
	STDMETHOD(start)();
	STDMETHOD(release)(BYTE diskid);
	STDMETHOD(mount)(BSTR path, BYTE diskid, VARIANT_BOOL readOnly);
	STDMETHOD(get_showDebugMessages)(VARIANT_BOOL* pVal);
	STDMETHOD(put_showDebugMessages)(VARIANT_BOOL newVal);
};

#endif //__Shell_H_
